import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:provider/provider.dart';
import 'package:image_picker/image_picker.dart';
import 'package:image_cropper/image_cropper.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';

import '../../theme/senior_theme_data.dart';
import './senior_image_cropper_style.dart';
import '../senior_button/senior_button.dart';
import '../senior_bottom_sheet/senior_bottom_sheet.dart';
import '../../repositories/theme_repository.dart';

class SeniorImageCropper {
  /// Creates the SDS Image Cropper component.
  /// The parameters [cropperTitle], [takePhotoLabel], [pickImageLabel],
  /// [onCropImage], [onPermissionCameraDenied] and [onPermissionGalleryDenied]
  /// are required.
  SeniorImageCropper({
    required this.cropperTitle,
    required this.takePhotoLabel,
    required this.pickImageLabel,
    this.doneButtonLabel,
    this.cancelButtonLabel,
    required this.onCropImage,
    this.maxSizeMb,
    this.onPermissionCameraDenied,
    this.onPermissionGalleryDenied,
    this.onException,
    this.style,
  });

  /// Title for the image cropper.
  final String cropperTitle;

  /// The label for the option to take a photo with your smartphone camera.
  final String takePhotoLabel;

  /// The label for the option to choose a saved image.
  final String pickImageLabel;

  /// The label for tue done button on iOS.
  final String? doneButtonLabel;

  /// The label for the cancel button on iOS.
  final String? cancelButtonLabel;

  /// Callback function executed when the image is cropped.
  /// Receive the cropped image.
  final Function(File croppedImage) onCropImage;

  /// Maximum image size.
  final int? maxSizeMb;

  /// Callback function executed when there is no permission to access the camera.
  final VoidCallback? onPermissionCameraDenied;

  /// Callback function executed when there is no permission to access the gallery.
  final VoidCallback? onPermissionGalleryDenied;

  /// Function executed when an unexpected exception happens.
  final Function(Object e)? onException;

  /// The component's style definitions.
  /// Allows you to configure?
  /// [SeniorImageCropperStyle.toolbarColor] the toolbar color of the crop view.
  /// [SeniorImageCropperStyle.toolbarContentColor] the toolbar content color of the crop view.
  /// [SeniorImageCropperStyle.activeControlsColor] the color of the active controls of the crop view.
  final SeniorImageCropperStyle? style;

  final _picker = ImagePicker();

  void pickImage(BuildContext context) {
    const double BUTTON_MAX_WIDTH = 600;
    final theme = Provider.of<ThemeRepository>(context, listen: false).theme;

    SeniorBottomSheet.showDynamicBottomSheet(
      context: context,
      isDismissible: true,
      hasCloseButton: false,
      content: [
        Container(
          child: Padding(
            padding: EdgeInsets.only(
              bottom:
                  SeniorSpacing.normal + MediaQuery.of(context).padding.bottom,
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: MainAxisSize.min,
              children: [
                SeniorButton.primary(
                  label: takePhotoLabel,
                  fullWidth:
                      MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                  onPressed: () {
                    _imgFromCamera(theme);
                    Navigator.pop(context);
                  },
                ),
                const SizedBox(
                  height: SeniorSpacing.medium,
                ),
                SeniorButton.secondary(
                  label: pickImageLabel,
                  fullWidth:
                      MediaQuery.of(context).size.width < BUTTON_MAX_WIDTH,
                  onPressed: () {
                    _imgFromGallery(theme);
                    Navigator.pop(context);
                  },
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }

  void _imgFromGallery(SeniorThemeData? theme) async {
    try {
      await _picker
          .pickImage(
        source: ImageSource.gallery,
        imageQuality: 50,
      )
          .then((value) {
        if (value != null) {
          _cropImage(imgFile: File(value.path), theme: theme);
        }
      });
    } on PlatformException catch (e) {
      if (e.code == 'camera_access_denied') {
        onPermissionGalleryDenied?.call();
      }
    } catch (e) {
      onException?.call(e);
    }
  }

  void _imgFromCamera(SeniorThemeData? theme) async {
    try {
      await _picker
          .pickImage(
        source: ImageSource.camera,
        imageQuality: 50,
      )
          .then((value) {
        if (value != null) {
          _cropImage(imgFile: File(value.path), theme: theme);
        }
      });
    } on PlatformException catch (e) {
      if (e.code == 'photo_access_denied') {
        onPermissionGalleryDenied?.call();
      }
    } catch (e) {
      onException?.call(e);
    }
  }

  void _cropImage({
    required File imgFile,
    required SeniorThemeData? theme,
  }) async {
    final croppedFile = await ImageCropper().cropImage(
      sourcePath: imgFile.path,
      uiSettings: [
        AndroidUiSettings(
          toolbarTitle: 'Image Cropper',
          toolbarColor: style?.toolbarColor ??
              theme?.imageCropperTheme?.style?.toolbarColor ??
              Colors.black,
          toolbarWidgetColor: style?.toolbarContentColor ??
              theme?.imageCropperTheme?.style?.toolbarContentColor ??
              SeniorColors.grayscale0,
          initAspectRatio: CropAspectRatioPreset.original,
          lockAspectRatio: false,
          showCropGrid: true,
          activeControlsWidgetColor: style?.activeControlsColor ??
              theme?.imageCropperTheme?.style?.activeControlsColor ??
              SeniorColors.primaryColor,
          hideBottomControls: true,
        ),
        IOSUiSettings(
          title: 'Image Cropper',
          doneButtonTitle: doneButtonLabel,
          cancelButtonTitle: cancelButtonLabel,
          hidesNavigationBar: true,
          resetButtonHidden: true,
          rotateButtonsHidden: true,
          rotateClockwiseButtonHidden: true,
          aspectRatioPickerButtonHidden: true,
        ),
      ],
    );

    if (croppedFile != null) {
      imageCache.clear();
      final File _croppedFile = File(croppedFile.path);
      _compressImage(_croppedFile);
      onCropImage(_croppedFile);
    }
  }

  void _compressImage(File file) async {
    if (maxSizeMb != null) {
      final maxLength = maxSizeMb! * 1024 * 1024;
      while (file.lengthSync() > maxLength) {
        final compressedImage = await FlutterImageCompress.compressWithList(
          file.readAsBytesSync(),
          quality: 50,
          format: CompressFormat.jpeg,
        );

        await file.writeAsBytes(
          compressedImage,
          flush: true,
          mode: FileMode.write,
        );
      }
    }
  }
}
