import 'package:flutter/material.dart';

import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_modal_style.dart';
import './model/senior_modal_action.dart';
import '../../components/senior_button/senior_button.dart';
import '../../repositories/theme_repository.dart';

class SeniorModal extends StatelessWidget {
  /// Displays a modal according to SDS definitions.
  ///
  /// The [content], [ghostButton] and [title] parameters are required.
  const SeniorModal({
    Key? key,
    required this.defaultAction,
    this.otherAction,
    required this.content,
    this.style,
    required this.title,
  }) : super(key: key);

  final SeniorModalAction defaultAction;
  final SeniorModalAction? otherAction;

  /// The information content that will be displayed in the modal.
  final String content;

  /// The component's style definitions.
  ///
  /// Allows you to configure:
  /// [SeniorModalStyle.backgroundColor] the background color of the modal.
  final SeniorModalStyle? style;

  /// The modal title.
  final String title;

  Widget _buildRowActionButtons() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        SeniorButton.ghost(
          label: defaultAction.label,
          onPressed: defaultAction.action,
          icon: defaultAction.icon,
        ),
        otherAction != null
            ? Padding(
                padding: const EdgeInsets.only(
                  left: SeniorSpacing.small,
                ),
                child: SeniorButton.primary(
                  label: otherAction!.label,
                  onPressed: otherAction!.action,
                  icon: otherAction!.icon,
                  danger: otherAction!.danger,
                ),
              )
            : const SizedBox.shrink(),
      ],
    );
  }

  Widget _buildColumnActionButtons() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        otherAction != null
            ? Padding(
                padding: const EdgeInsets.only(bottom: SeniorSpacing.small),
                child: SeniorButton.primary(
                  label: otherAction!.label,
                  onPressed: otherAction!.action,
                  icon: otherAction!.icon,
                  danger: otherAction!.danger,
                  fullWidth: true,
                ),
              )
            : const SizedBox.shrink(),
        SeniorButton.ghost(
          label: defaultAction.label,
          onPressed: defaultAction.action,
          icon: defaultAction.icon,
          fullWidth: true,
        ),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    return AlertDialog(
      surfaceTintColor: Colors.transparent,
      backgroundColor: style?.backgroundColor ?? theme.modalTheme?.style?.backgroundColor ?? SeniorColors.pureWhite,
      title: Text(
        title,
        style: SeniorTypography.h3(
          color: style?.titleColor ?? theme.modalTheme?.style?.titleColor ?? SeniorColors.neutralColor800,
        ),
      ),
      titlePadding: const EdgeInsets.only(
        top: SeniorSpacing.normal,
        right: SeniorSpacing.normal,
        left: SeniorSpacing.normal,
        bottom: SeniorSpacing.xsmall,
      ),
      contentPadding: const EdgeInsets.only(
        left: SeniorSpacing.normal,
        right: SeniorSpacing.normal,
        bottom: SeniorSpacing.normal,
      ),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Flexible(
            child: SingleChildScrollView(
              child: Text(
                content,
                style: SeniorTypography.label(
                  color: style?.contentColor ?? theme.modalTheme?.style?.contentColor ?? SeniorColors.secondaryColor700,
                ),
              ),
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(top: SeniorSpacing.small),
            child: MediaQuery.of(context).size.width >= 576 ? _buildRowActionButtons() : _buildColumnActionButtons(),
          ),
        ],
      ),
    );
  }
}
